
#pragma once

#include <dune/phasefield/base_parameters.hh>


template <typename Number, typename TW, typename Mobility, typename Reaction, typename VelDissipation>
class Params_thinstrip : public Params_ffs_r<Number, TW, Mobility, Reaction, VelDissipation>
{
public:
  const Number YScaling;
  const Number len;
  Number gradP;
  const Number ionSource;

  Params_thinstrip(const Dune::ParameterTree& ptree) :
    Params_ffs_r<Number, TW, Mobility, Reaction, VelDissipation>(ptree),
    YScaling(ptree.get("domain.YScaling",1.0)),
    len(ptree.get("domain.len",1.0)),
    gradP(ptree.get("Initial.pConst",1.0)),
    ionSource(ptree.get("Initial.ionSource",1.0))
  {
    ModifyParameters();
    std::cout << "YScaling " << YScaling << " Eps " << this->eps << std::endl;
  }

  void ModifyParameters() //We are in big (i.e. xref) coordinates!
  {
    this->gradP *= 1./(YScaling*YScaling);
    this->eps *= YScaling;
    this->mobility.mobility *= YScaling*YScaling;
    this->reaction.reactionRate *= YScaling;
    this->vDis.velDissipationRate *= 1./(YScaling*YScaling);
  }
};


template<typename Number>
class Reaction_Thinstrip
{
public:
  Number reactionRate;
  Reaction_Thinstrip(const Dune::ParameterTree & param) :
    reactionRate(param.get("ReactionRate",(Number)0.1))
  {
  }

  Number eval( const Number& u) const
  {
    if(u<0)
      return reactionRate*(0-0.5);
    else
      return reactionRate*(u-0.5);
  }
};
