#pragma once

#include<dune/pdelab/common/quadraturerules.hh>
#include<dune/pdelab/finiteelement/localbasiscache.hh>
#include<dune/pdelab/localoperator/defaultimp.hh>
#include<dune/pdelab/localoperator/pattern.hh>
#include<dune/pdelab/localoperator/flags.hh>
#include<dune/pdelab/localoperator/variablefactories.hh>
#include<dune/pdelab/gridfunctionspace/gridfunctionspace.hh>





template<typename Param, typename FEM, typename RF>
    class TMIXED_UpscaledP :
      public Dune::PDELab::FullVolumePattern,
      public Dune::PDELab::LocalOperatorDefaultFlags,
      public Dune::PDELab::NumericalJacobianVolume<TMIXED_UpscaledP<Param, FEM, RF> >,
      public Dune::PDELab::NumericalJacobianApplyVolume<TMIXED_UpscaledP<Param, FEM, RF> >
    {
    private:
      //typedef typename CHContainer::field_type RF;
      //typedef typename NSGFS::Traits::GridViewType::template Codim<0>::Entity   ENTITY;

      typedef typename FEM::Traits::FiniteElementType::Traits::LocalBasisType LocalBasis;
      Dune::PDELab::LocalBasisCache<LocalBasis> cache;

      Param& param; // parameter functions


    public:
      // pattern assembly flags
      enum { doPatternVolume = true };

      // residual assembly flags
      enum { doAlphaVolume = true };
      //enum { doLambdaVolume = true };



      TMIXED_UpscaledP (Param& param_) :
        param(param_)
      {
      }


      template<typename EG, typename LFSU, typename X, typename LFSV, typename R>
      void alpha_volume (const EG& eg, const LFSU& lfsu, const X& x, const LFSV& lfsv, R& r) const
      {
//        std::cout << "hi_alpha" << std::endl;
        //Quadrature Rule
        const int dim = EG::Entity::dimension;
        const int order = 2; //TODO: Choose more educated
        auto rule = Dune::PDELab::quadratureRule(eg.geometry(), order);

        for(const auto& ip : rule)
        {
          // evaluate shape functions and  gradient of shape functions
          auto& phi = cache.evaluateFunction(ip.position(), lfsv.finiteElement().localBasis());
          auto& gradphihat = cache.evaluateJacobian(ip.position(), lfsv.finiteElement().localBasis());

          // transform gradients of shape functions to real element
          const auto S = eg.geometry().jacobianInverseTransposed(ip.position());
          auto gradphi = makeJacobianContainer(lfsv);
          for (size_t i=0; i<lfsv.size(); i++)
          {
            S.mv(gradphihat[i][0],gradphi[i][0]);
          }
          RF p=0.0;
          Dune::FieldVector<RF,dim> gradp(0.0);
          for (size_t i=0; i<lfsu.size(); i++)
          {
            gradp.axpy(x(lfsv,i),gradphi[i][0]);
            p += x(lfsv,i)*phi[i];
          }
          auto xg = eg.geometry().global(ip.position());
          gradp[0] = gradp[0] - 0.05; //TODO: Make more general for other pressure jumps/distances
          //auto xg = eg.geometry().center();
          //std::cout << "pos: " << xg[0] << " kf: " << kfresult-kfold << std::endl;
          auto factor = ip.weight()*eg.geometry().integrationElement(ip.position());
          for (size_t i=0; i<lfsv.size(); i++)
          {
            r.accumulate(lfsv,i,( (gradp*gradphi[i][0])*param.evalKf(xg) )*factor);
          } // for i
        } // for ip
//        std::cout << "bye_alpha" << std::endl;
      } //alpha volume

    };
